@@ -8,7 +8,6 @@ import os
 import re
 import socket
 import logging
-import time
 import urlparse
 
 # import third party libs
@@ -24,7 +23,6 @@ import salt.crypt
 import salt.loader
 import salt.utils
 import salt.pillar
-from salt.exceptions import SaltClientError
 
 log = logging.getLogger(__name__)
 
@@ -35,6 +33,165 @@ _DFLT_LOG_FMT_LOGFILE = (
     '%(asctime)s,%(msecs)03.0f [%(name)-17s][%(levelname)-8s] %(message)s'
 )
 
+# default configurations
+DEFAULT_MINION_OPTS = {
+    'master': 'salt',
+    'master_port': '4506',
+    'master_finger': '',
+    'user': 'root',
+    'root_dir': '/',
+    'pki_dir': '/etc/salt/pki/minion',
+    'id': socket.getfqdn(),
+    'cachedir': '/var/cache/salt/minion',
+    'cache_jobs': False,
+    'conf_file': '/etc/salt/minion',
+    'sock_dir': '/var/run/salt/minion',
+    'backup_mode': '',
+    'renderer': 'yaml_jinja',
+    'failhard': False,
+    'autoload_dynamic_modules': True,
+    'environment': None,
+    'state_top': 'top.sls',
+    'startup_states': '',
+    'sls_list': [],
+    'top_file': '',
+    'file_client': 'remote',
+    'file_roots': {
+        'base': ['/srv/salt'],
+        },
+    'pillar_roots': {
+        'base': ['/srv/pillar'],
+        },
+    'hash_type': 'md5',
+    'external_nodes': '',
+    'disable_modules': [],
+    'disable_returners': [],
+    'whitelist_modules': [],
+    'module_dirs': [],
+    'returner_dirs': [],
+    'states_dirs': [],
+    'render_dirs': [],
+    'outputter_dirs': [],
+    'providers': {},
+    'clean_dynamic_modules': True,
+    'open_mode': False,
+    'multiprocessing': True,
+    'ipc_mode': 'ipc',
+    'tcp_pub_port': 4510,
+    'tcp_pull_port': 4511,
+    'log_file': '/var/log/salt/minion',
+    'log_level': None,
+    'log_level_logfile': None,
+    'log_datefmt': _DFLT_LOG_DATEFMT,
+    'log_datefmt_logfile': _DFLT_LOG_DATEFMT_LOGFILE,
+    'log_fmt_console': _DFLT_LOG_FMT_CONSOLE,
+    'log_fmt_logfile': _DFLT_LOG_FMT_LOGFILE,
+    'log_granular_levels': {},
+    'test': False,
+    'cython_enable': False,
+    'state_verbose': True,
+    'state_output': 'full',
+    'acceptance_wait_time': 10,
+    'loop_interval': 60,
+    'dns_check': True,
+    'verify_env': True,
+    'grains': {},
+    'permissive_pki_access': False,
+    'default_include': 'minion.d/*.conf',
+    'update_url': False,
+    'update_restart_services': [],
+    'retry_dns': 30,
+    'recon_max': 5000,
+    'win_repo_cachefile': 'salt://win/repo/winrepo.p',
+    'pidfile': '/var/run/salt-minion.pid',
+    'tcp_keepalive': True,
+    'tcp_keepalive_idle': 300,
+    'tcp_keepalive_cnt': -1,
+    'tcp_keepalive_intvl': -1,
+}
+
+DEFAULT_MASTER_OPTS = {
+    'interface': '0.0.0.0',
+    'publish_port': '4505',
+    'auth_mode': 1,
+    'user': 'root',
+    'worker_threads': 5,
+    'sock_dir': '/var/run/salt/master',
+    'ret_port': '4506',
+    'timeout': 5,
+    'keep_jobs': 24,
+    'root_dir': '/',
+    'pki_dir': '/etc/salt/pki/master',
+    'cachedir': '/var/cache/salt/master',
+    'file_roots': {
+        'base': ['/srv/salt'],
+        },
+    'master_roots': {
+        'base': ['/srv/salt-master'],
+        },
+    'pillar_roots': {
+        'base': ['/srv/pillar'],
+        },
+    'gitfs_remotes': [],
+    'ext_pillar': [],
+    'pillar_version': 2,
+    'pillar_opts': True,
+    'syndic_master': '',
+    'runner_dirs': [],
+    'outputter_dirs': [],
+    'client_acl': {},
+    'client_acl_blacklist': {},
+    'external_auth': {},
+    'token_expire': 43200,
+    'file_buffer_size': 1048576,
+    'file_ignore_regex': None,
+    'file_ignore_glob': None,
+    'fileserver_backend': ['roots'],
+    'max_open_files': 100000,
+    'hash_type': 'md5',
+    'conf_file': '/etc/salt/master',
+    'open_mode': False,
+    'auto_accept': False,
+    'renderer': 'yaml_jinja',
+    'failhard': False,
+    'state_top': 'top.sls',
+    'master_tops': {},
+    'external_nodes': '',
+    'order_masters': False,
+    'job_cache': True,
+    'ext_job_cache': '',
+    'master_ext_job_cache': '',
+    'minion_data_cache': True,
+    'log_file': '/var/log/salt/master',
+    'log_level': None,
+    'log_level_logfile': None,
+    'log_datefmt': _DFLT_LOG_DATEFMT,
+    'log_datefmt_logfile': _DFLT_LOG_DATEFMT_LOGFILE,
+    'log_fmt_console': _DFLT_LOG_FMT_CONSOLE,
+    'log_fmt_logfile': _DFLT_LOG_FMT_LOGFILE,
+    'log_granular_levels': {},
+    'pidfile': '/var/run/salt-master.pid',
+    'cluster_masters': [],
+    'cluster_mode': 'paranoid',
+    'range_server': 'range:80',
+    'reactors': [],
+    'serial': 'msgpack',
+    'state_verbose': True,
+    'state_output': 'full',
+    'search': '',
+    'search_index_interval': 3600,
+    'loop_interval': 60,
+    'nodegroups': {},
+    'cython_enable': False,
+    'key_logfile': '/var/log/salt/key',
+    'verify_env': True,
+    'permissive_pki_access': False,
+    'default_include': 'master.d/*.conf',
+    'win_repo': '/srv/salt/win/repo',
+    'win_repo_mastercachefile': '/srv/salt/win/repo/winrepo.p',
+    'win_gitrepos': ['https://github.com/saltstack/salt-winrepo.git'],
+}
+
 
 def _validate_file_roots(opts):
     '''
@@ -77,15 +234,15 @@ def _read_conf_file(path):
         return conf_opts
 
 
-def load_config(opts, path, env_var):
+def load_config(path, env_var):
     '''
-    Attempts to update ``opts`` dict by parsing either the file described by
+    Returns configuration dict from parsing either the file described by
     ``path`` or the environment variable described by ``env_var`` as YAML.
     '''
     if path is None:
         # When the passed path is None, we just want the configuration
         # defaults, not actually loading the whole configuration.
-        return opts
+        return {}
 
     if not path or not os.path.isfile(path):
         path = os.environ.get(env_var, path)
@@ -102,8 +259,9 @@ def load_config(opts, path, env_var):
 
     if os.path.isfile(path):
         try:
-            opts.update(_read_conf_file(path))
+            opts = _read_conf_file(path)
             opts['conf_file'] = path
+            return opts
         except Exception as err:
             import salt.log
             msg = 'Error parsing configuration file: {0} - {1}'
@@ -114,24 +272,27 @@ def load_config(opts, path, env_var):
     else:
         log.debug('Missing configuration file: {0}'.format(path))
 
+    return {}
 
-def include_config(include, opts, orig_path, verbose):
+
+def include_config(include, orig_path, verbose):
     '''
     Parses extra configuration file(s) specified in an include list in the
     main config file.
     '''
     # Protect against empty option
     if not include:
-        return opts
+        return {}
 
     if orig_path is None:
         # When the passed path is None, we just want the configuration
         # defaults, not actually loading the whole configuration.
-        return opts
+        return {}
 
     if isinstance(include, str):
         include = [include]
 
+    include_config = {}
     for path in include:
         if not os.path.isabs(path):
             path = os.path.join(os.path.dirname(orig_path), path)
@@ -147,14 +308,14 @@ def include_config(include, opts, orig_path, verbose):
 
         for fn_ in glob.glob(path):
             try:
-                opts.update(_read_conf_file(fn_))
+                include_config.update(_read_conf_file(fn_))
             except Exception as err:
                 log.warn(
                     'Error parsing configuration file: {0} - {1}'.format(
                         fn_, err
                     )
                 )
-    return opts
+    return include_config
 
 
 def prepend_root_dir(opts, path_options):
@@ -168,135 +329,49 @@ def prepend_root_dir(opts, path_options):
             if opts[path_option].startswith(opts['root_dir']):
                 opts[path_option] = opts[path_option][len(opts['root_dir']):]
             opts[path_option] = salt.utils.path_join(
-                    root_dir,
-                    opts[path_option])
+                root_dir,
+                opts[path_option]
+            )
 
 
-def minion_config(path, check_dns=True):
+def minion_config(path,
+                  check_dns=True,
+                  env_var='SALT_MINION_CONFIG',
+                  defaults=None):
     '''
     Reads in the minion configuration file and sets up special options
     '''
-    opts = {'master': 'salt',
-            'master_port': '4506',
-            'master_finger': '',
-            'user': 'root',
-            'root_dir': '/',
-            'pki_dir': '/etc/salt/pki/minion',
-            'id': socket.getfqdn(),
-            'cachedir': '/var/cache/salt/minion',
-            'cache_jobs': False,
-            'conf_file': path,
-            'sock_dir': '/var/run/salt/minion',
-            'backup_mode': '',
-            'renderer': 'yaml_jinja',
-            'failhard': False,
-            'autoload_dynamic_modules': True,
-            'environment': None,
-            'state_top': 'top.sls',
-            'startup_states': '',
-            'sls_list': [],
-            'top_file': '',
-            'file_client': 'remote',
-            'file_roots': {
-                'base': ['/srv/salt'],
-                },
-            'pillar_roots': {
-                'base': ['/srv/pillar'],
-                },
-            'hash_type': 'md5',
-            'external_nodes': '',
-            'disable_modules': [],
-            'disable_returners': [],
-            'whitelist_modules': [],
-            'module_dirs': [],
-            'returner_dirs': [],
-            'states_dirs': [],
-            'render_dirs': [],
-            'providers': {},
-            'clean_dynamic_modules': True,
-            'open_mode': False,
-            'multiprocessing': True,
-            'ipc_mode': 'ipc',
-            'tcp_pub_port': 4510,
-            'tcp_pull_port': 4511,
-            'log_file': '/var/log/salt/minion',
-            'log_level': None,
-            'log_level_logfile': None,
-            'log_datefmt': _DFLT_LOG_DATEFMT,
-            'log_datefmt_logfile': _DFLT_LOG_DATEFMT_LOGFILE,
-            'log_fmt_console': _DFLT_LOG_FMT_CONSOLE,
-            'log_fmt_logfile': _DFLT_LOG_FMT_LOGFILE,
-            'log_granular_levels': {},
-            'test': False,
-            'cython_enable': False,
-            'state_verbose': True,
-            'state_output': 'full',
-            'acceptance_wait_time': 10,
-            'loop_interval': 60,
-            'dns_check': True,
-            'verify_env': True,
-            'grains': {},
-            'permissive_pki_access': False,
-            'default_include': 'minion.d/*.conf',
-            'update_url': False,
-            'update_restart_services': [],
-            'retry_dns': 30,
-            'recon_max': 5000,
-            'win_repo_cachefile': 'salt://win/repo/winrepo.p',
-            'pidfile': '/var/run/salt-minion.pid',
-            'tcp_keepalive': True,
-            'tcp_keepalive_idle': 300,
-            'tcp_keepalive_cnt': -1,
-            'tcp_keepalive_intvl': -1,
-            }
+    if defaults is None:
+        defaults = DEFAULT_MINION_OPTS
+
+    overrides = load_config(path, env_var)
+    default_include = overrides.get('default_include',
+                                    defaults['default_include'])
+    include = overrides.get('include', [])
+
+    overrides.update(include_config(default_include, path, verbose=False))
+    overrides.update(include_config(include, path, verbose=True))
+
+    return apply_minion_config(overrides, check_dns, defaults)
+
+
+def apply_minion_config(overrides=None, check_dns=True, defaults=None):
+    '''
+    Returns minion configurations dict.
+    '''
+    if defaults is None:
+        defaults = DEFAULT_MINION_OPTS
+
+    opts = defaults.copy()
+    if overrides:
+        opts.update(overrides)
 
     if len(opts['sock_dir']) > len(opts['cachedir']) + 10:
         opts['sock_dir'] = os.path.join(opts['cachedir'], '.salt-unix')
 
-    load_config(opts, path, 'SALT_MINION_CONFIG')
-
-    default_include = opts.get('default_include', [])
-    include = opts.get('include', [])
-
-    opts = include_config(default_include, opts, path, verbose=False)
-    opts = include_config(include, opts, path, verbose=True)
-
     if 'append_domain' in opts:
         opts['id'] = _append_domain(opts)
 
-    if opts.get('file_client', 'remote') == 'local' and check_dns:
-        check_dns = False
-
-    if check_dns is True:
-        # Because I import salt.log below I need to re-import salt.utils here
-        import salt.utils
-        try:
-            opts['master_ip'] = salt.utils.dns_check(opts['master'], True)
-        except SaltClientError:
-            if opts['retry_dns']:
-                while True:
-                    import salt.log
-                    msg = ('Master hostname: {0} not found. Retrying in {1} '
-                           'seconds').format(opts['master'], opts['retry_dns'])
-                    if salt.log.is_console_configured():
-                        log.warn(msg)
-                    else:
-                        print('WARNING: {0}'.format(msg))
-                    time.sleep(opts['retry_dns'])
-                    try:
-                        opts['master_ip'] = salt.utils.dns_check(
-                            opts['master'], True
-                        )
-                        break
-                    except SaltClientError:
-                        pass
-            else:
-                opts['master_ip'] = '127.0.0.1'
-    else:
-        opts['master_ip'] = '127.0.0.1'
-
-    opts['master_uri'] = 'tcp://{ip}:{port}'.format(ip=opts['master_ip'],
-                                                    port=opts['master_port'])
 
     # Enabling open mode requires that the value be set to True, and
     # nothing else!
@@ -304,9 +379,9 @@ def minion_config(path, check_dns=True):
 
     # set up the extension_modules location from the cachedir
     opts['extension_modules'] = (
-            opts.get('extension_modules') or
-            os.path.join(opts['cachedir'], 'extmods')
-            )
+        opts.get('extension_modules') or
+        os.path.join(opts['cachedir'], 'extmods')
+    )
 
     # Prepend root_dir to other paths
     prepend_root_dirs = [
@@ -322,106 +397,43 @@ def minion_config(path, check_dns=True):
     return opts
 
 
-def master_config(path):
+def master_config(path, env_var='SALT_MASTER_CONFIG', defaults=None):
     '''
     Reads in the master configuration file and sets up default options
     '''
-    opts = {'interface': '0.0.0.0',
-            'publish_port': '4505',
-            'auth_mode': 1,
-            'user': 'root',
-            'worker_threads': 5,
-            'sock_dir': '/var/run/salt/master',
-            'ret_port': '4506',
-            'timeout': 5,
-            'keep_jobs': 24,
-            'root_dir': '/',
-            'pki_dir': '/etc/salt/pki/master',
-            'cachedir': '/var/cache/salt/master',
-            'file_roots': {
-                'base': ['/srv/salt'],
-                },
-            'master_roots': {
-                'base': ['/srv/salt-master'],
-                },
-            'pillar_roots': {
-                'base': ['/srv/pillar'],
-                },
-            'gitfs_remotes': [],
-            'ext_pillar': [],
-            'pillar_version': 2,
-            'pillar_opts': True,
-            'syndic_master': '',
-            'runner_dirs': [],
-            'client_acl': {},
-            'external_auth': {},
-            'token_expire': 43200,
-            'file_buffer_size': 1048576,
-            'file_ignore_regex': None,
-            'file_ignore_glob': None,
-            'fileserver_backend': ['roots'],
-            'max_open_files': 100000,
-            'hash_type': 'md5',
-            'conf_file': path,
-            'open_mode': False,
-            'auto_accept': False,
-            'renderer': 'yaml_jinja',
-            'failhard': False,
-            'state_top': 'top.sls',
-            'master_tops': {},
-            'external_nodes': '',
-            'order_masters': False,
-            'job_cache': True,
-            'ext_job_cache': '',
-            'master_ext_job_cache': '',
-            'minion_data_cache': True,
-            'log_file': '/var/log/salt/master',
-            'log_level': None,
-            'log_level_logfile': None,
-            'log_datefmt': _DFLT_LOG_DATEFMT,
-            'log_datefmt_logfile': _DFLT_LOG_DATEFMT_LOGFILE,
-            'log_fmt_console': _DFLT_LOG_FMT_CONSOLE,
-            'log_fmt_logfile': _DFLT_LOG_FMT_LOGFILE,
-            'log_granular_levels': {},
-            'pidfile': '/var/run/salt-master.pid',
-            'cluster_masters': [],
-            'cluster_mode': 'paranoid',
-            'range_server': 'range:80',
-            'reactors': [],
-            'serial': 'msgpack',
-            'state_verbose': True,
-            'state_output': 'full',
-            'search': '',
-            'search_index_interval': 3600,
-            'loop_interval': 60,
-            'nodegroups': {},
-            'cython_enable': False,
-            'key_logfile': '/var/log/salt/key',
-            'verify_env': True,
-            'permissive_pki_access': False,
-            'default_include': 'master.d/*.conf',
-            'win_repo': '/srv/salt/win/repo',
-            'win_repo_mastercachefile': '/srv/salt/win/repo/winrepo.p',
-            'win_gitrepos': ['https://github.com/saltstack/salt-winrepo.git'],
-    }
+    if defaults is None:
+        defaults = DEFAULT_MASTER_OPTS
+
+    overrides = load_config(path, env_var)
+    default_include = overrides.get('default_include',
+                                    defaults['default_include'])
+    include = overrides.get('include', [])
+
+    overrides.update(include_config(default_include, path, verbose=False))
+    overrides.update(include_config(include, path, verbose=True))
+    return apply_master_config(overrides, defaults)
+
+
+def apply_master_config(overrides=None, defaults=None):
+    '''
+    Returns master configurations dict.
+    '''
+    if defaults is None:
+        defaults = DEFAULT_MASTER_OPTS
+
+    opts = defaults.copy()
+    if overrides:
+        opts.update(overrides)
 
     if len(opts['sock_dir']) > len(opts['cachedir']) + 10:
         opts['sock_dir'] = os.path.join(opts['cachedir'], '.salt-unix')
 
-    load_config(opts, path, 'SALT_MASTER_CONFIG')
-
-    default_include = opts.get('default_include', [])
-    include = opts.get('include', [])
-
-    opts = include_config(default_include, opts, path, verbose=False)
-    opts = include_config(include, opts, path, verbose=True)
-
     opts['aes'] = salt.crypt.Crypticle.generate_key_string()
 
     opts['extension_modules'] = (
-            opts.get('extension_modules') or
-            os.path.join(opts['cachedir'], 'extmods')
-            )
+        opts.get('extension_modules') or
+        os.path.join(opts['cachedir'], 'extmods')
+    )
     opts['token_dir'] = os.path.join(opts['cachedir'], 'tokens')
 
     # Prepend root_dir to other paths
@@ -432,7 +444,11 @@ def master_config(path):
 
     # These can be set to syslog, so, not actual paths on the system
     for config_key in ('log_file', 'key_logfile'):
-        if urlparse.urlparse(opts.get(config_key, '')).scheme == '':
+        log_setting = opts.get(config_key, '')
+        if log_setting is None:
+            continue
+
+        if urlparse.urlparse(log_setting).scheme == '':
             prepend_root_dirs.append(config_key)
 
     prepend_root_dir(opts, prepend_root_dirs)
@@ -473,19 +489,44 @@ def master_config(path):
     return opts
 
 
-def client_config(path):
+def client_config(path, env_var='SALT_CLIENT_CONFIG', defaults=None):
     '''
     Load in the configuration data needed for the LocalClient. This function
     searches for client specific configurations and adds them to the data from
     the master configuration.
     '''
-    opts = {'token_file': os.path.expanduser('~/.salt_token')}
-    opts.update(master_config(path))
-    cpath = os.path.expanduser('~/.salt')
-    load_config(opts, cpath, 'SALT_CLIENT_CONFIG')
+    if defaults is None:
+        defaults = DEFAULT_MASTER_OPTS
+
+    # Get the token file path from the provided defaults. If not found, specify
+    # our own, sane, default
+    opts = {
+        'token_file': defaults.get(
+            'token_file',
+            os.path.expanduser('~/.salt_token')
+        )
+    }
+    # Update options with the master configuration, either from the provided
+    # path, salt's defaults or provided defaults
+    opts.update(
+        master_config(path, defaults=defaults)
+    )
+    # Update with the users salt dot file or with the environment variable
+    opts.update(
+        load_config(
+            os.path.expanduser('~/.salt'), env_var
+        )
+    )
+    # Make sure we have a proper and absolute path to the token file
     if 'token_file' in opts:
-        opts['token_file'] = os.path.expanduser(opts['token_file'])
+        opts['token_file'] = os.path.abspath(
+            os.path.expanduser(
+                opts['token_file']
+            )
+        )
+    # If the token file exists, read and store the contained token
     if os.path.isfile(opts['token_file']):
         with salt.utils.fopen(opts['token_file']) as fp_:
             opts['token'] = fp_.read().strip()
+    # Return the client options
     return opts
