@@ -7,7 +7,7 @@ data
 # some time in the future
 
 # Import python libs
-from contextlib import nested  # For < 2.7 compat
+import contextlib  # For < 2.7 compat
 import os
 import re
 import time
@@ -29,9 +29,9 @@ except ImportError:
 # Import salt libs
 import salt.utils
 import salt.utils.find
-from salt.utils.filebuffer import BufferedReader
+import salt.utils.filebuffer
 from salt.exceptions import CommandExecutionError, SaltInvocationError
-from salt._compat import string_types, urlparse
+import salt._compat
 
 
 def __virtual__():
@@ -39,17 +39,11 @@ def __virtual__():
     Only work on posix-like systems
     '''
     # win_file takes care of windows
-    if __grains__['os'] == 'Windows':
+    if salt.utils.is_windows():
         return False
     return 'file'
 
 
-__outputter__ = {
-    'touch':  'txt',
-    'append': 'txt',
-}
-
-
 def __clean_tmp(sfn):
     '''
     Clean out a template temp file
@@ -71,8 +65,8 @@ def _is_bin(path):
     Return True if a file is a bin, just checks for NULL char, this should be
     expanded to reflect how git checks for bins
     '''
-    with salt.utils.fopen(path, 'rb') as f:
-        return '\0' in f.read(2048)
+    with salt.utils.fopen(path, 'rb') as ifile:
+        return '\0' in ifile.read(2048)
 
 
 def gid_to_group(gid):
@@ -287,16 +281,16 @@ def get_sum(path, form='md5'):
     if not os.path.isfile(path):
         return 'File not found'
     try:
-        with salt.utils.fopen(path, 'rb') as f:
-            return getattr(hashlib, form)(f.read()).hexdigest()
-    except (IOError, OSError) as e:
-        return 'File Error: {0}'.format(e)
+        with salt.utils.fopen(path, 'rb') as ifile:
+            return getattr(hashlib, form)(ifile.read()).hexdigest()
+    except (IOError, OSError) as err:
+        return 'File Error: {0}'.format(err)
     except AttributeError:
         return 'Hash {0} not supported'.format(form)
     except NameError:
         return 'Hashlib unavailable - please fix your python install'
-    except Exception as e:
-        return str(e)
+    except Exception as err:
+        return str(err)
 
 
 def get_hash(path, form='md5', chunk_size=4096):
@@ -308,15 +302,19 @@ def get_hash(path, form='md5', chunk_size=4096):
         - It does not return a string on error. The returned value of
             ``get_sum`` cannot really be trusted since it is vulnerable to
             collisions: ``get_sum(..., 'xyz') == 'Hash xyz not supported'``
+
+    CLI Example::
+
+        salt '*' file.get_hash /etc/shadow
     '''
     try:
         hash_type = getattr(hashlib, form)
     except AttributeError:
         raise ValueError('Invalid hash type: {0}'.format(form))
-    with salt.utils.fopen(path, 'rb') as f:
+    with salt.utils.fopen(path, 'rb') as ifile:
         hash_obj = hash_type()
         while True:
-            chunk = f.read(chunk_size)
+            chunk = ifile.read(chunk_size)
             if not chunk:
                 return hash_obj.hexdigest()
             hash_obj.update(chunk)
@@ -334,6 +332,10 @@ def check_hash(path, hash):
     hash
         A string in the form <hash_type>=<hash_value>. For example:
         ``md5=e138491e9d5b97023cea823fe17bac22``
+
+    CLI Example::
+
+        salt '*' file.check_hash /etc/fstab md5=<md5sum>
     '''
     hash_parts = hash.split('=', 1)
     if len(hash_parts) != 2:
@@ -434,25 +436,25 @@ def find(path, **kwargs):
         salt '*' file.find /var/log name=\*.[0-9] mtime=+30d size=+10m delete
     '''
     try:
-        f = salt.utils.find.Finder(kwargs)
+        finder = salt.utils.find.Finder(kwargs)
     except ValueError as ex:
         return 'error: {0}'.format(ex)
 
-    ret = [p for p in f.find(path)]
+    ret = [p for p in finder.find(path)]
     ret.sort()
     return ret
 
 
-def _sed_esc(s, escape_all=False):
+def _sed_esc(string, escape_all=False):
     '''
     Escape single quotes and forward slashes
     '''
     special_chars = "^.[$()|*+?{"
-    s = s.replace("'", "'\"'\"'").replace("/", "\/")
+    string = string.replace("'", "'\"'\"'").replace("/", "\/")
     if escape_all:
-        for ch in special_chars:
-            s = s.replace(ch, "\\" + ch)
-    return s
+        for char in special_chars:
+            string = string.replace(char, "\\" + char)
+    return string
 
 
 def sed(path, before, after, limit='', backup='.bak', options='-r -e',
@@ -637,10 +639,11 @@ def contains(path, text):
     if not os.path.exists(path):
         return False
 
+    stripped_text = text.strip()
     try:
-        with BufferedReader(path) as br:
-            for chunk in br:
-                if text.strip() == chunk.strip():
+        with salt.utils.filebuffer.BufferedReader(path) as breader:
+            for chunk in breader:
+                if stripped_text in chunk:
                     return True
         return False
     except (IOError, OSError):
@@ -654,14 +657,14 @@ def contains_regex(path, regex, lchar=''):
 
     CLI Example::
 
-        salt '*' /etc/crontab '^maint'
+        salt '*' file.contains_regex /etc/crontab '^maint'
     '''
     if not os.path.exists(path):
         return False
 
     try:
-        with BufferedReader(path) as br:
-            for chunk in br:
+        with salt.utils.filebuffer.BufferedReader(path) as breader:
+            for chunk in breader:
                 if lchar:
                     chunk = chunk.lstrip(lchar)
                 if re.search(regex, chunk, re.MULTILINE):
@@ -677,14 +680,14 @@ def contains_glob(path, glob):
 
     CLI Example::
 
-        salt '*' /etc/foobar '*cheese*'
+        salt '*' file.contains_glob /etc/foobar '*cheese*'
     '''
     if not os.path.exists(path):
         return False
 
     try:
-        with BufferedReader(path) as br:
-            for chunk in br:
+        with salt.utils.filebuffer.BufferedReader(path) as breader:
+            for chunk in breader:
                 if fnmatch.fnmatch(chunk, glob):
                     return True
             return False
@@ -706,9 +709,9 @@ def append(path, *args):
     '''
     # Largely inspired by Fabric's contrib.files.append()
 
-    with salt.utils.fopen(path, "a") as f:
+    with salt.utils.fopen(path, "a") as ofile:
         for line in args:
-            f.write('{0}\n'.format(line))
+            ofile.write('{0}\n'.format(line))
 
     return 'Wrote {0} lines to "{1}"'.format(len(args), path)
 
@@ -802,6 +805,13 @@ def stats(path, hash_type='md5', follow_symlink=False):
 
 
 def remove(path):
+    '''
+    Remove the named file
+
+    CLI Example::
+    
+        salt '*' file.remove /tmp/foo
+    '''
     if not os.path.isabs(path):
         raise SaltInvocationError('File path must be absolute.')
 
@@ -847,7 +857,7 @@ def restorecon(path, recursive=False):
 
     CLI Example::
 
-         salt '*' selinux.restorecon /home/user/.ssh/authorized_keys
+         salt '*' file.restorecon /home/user/.ssh/authorized_keys
     '''
     if recursive:
         cmd = 'restorecon -FR {0}'.format(path)
@@ -862,7 +872,7 @@ def get_selinux_context(path):
 
     CLI Example::
 
-        salt '*' selinux.get_context /etc/hosts
+        salt '*' file.get_selinux_context /etc/hosts
     '''
     out = __salt__['cmd.run']('ls -Z {0}'.format(path))
     return out.split(' ')[4]
@@ -874,7 +884,7 @@ def set_selinux_context(path, user=None, role=None, type=None, range=None):
 
     CLI Example::
 
-        salt '*' selinux.chcon path <role> <type> <range>
+        salt '*' file.set_selinux_context path <role> <type> <range>
     '''
     if not user and not role and not type and not range:
         return False
@@ -900,6 +910,9 @@ def set_selinux_context(path, user=None, role=None, type=None, range=None):
 def source_list(source, source_hash, env):
     '''
     Check the source list and return the source to use
+
+    CLI Example::
+        salt '*' file.source_list salt://http/httpd.conf '{hash_type: 'md5', 'hsum': <md5sum>}' base
     '''
     if isinstance(source, list):
         # get the master file list
@@ -913,7 +926,7 @@ def source_list(source, source_hash, env):
                     continue
                 single_src = next(iter(single))
                 single_hash = single[single_src]
-                proto = urlparse(single_src).scheme
+                proto = salt._compat.urlparse(single_src).scheme
                 if proto == 'salt':
                     if single_src in mfiles:
                         source = single_src
@@ -926,7 +939,7 @@ def source_list(source, source_hash, env):
                         source = single_src
                         source_hash = single_hash
                         break
-            elif isinstance(single, string_types):
+            elif isinstance(single, salt._compat.string_types):
                 if single[7:] in mfiles or single[7:] in mdirs:
                     source = single
                     break
@@ -947,6 +960,10 @@ def get_managed(
         **kwargs):
     '''
     Return the managed file data for file.managed
+
+    CLI Example::
+
+        salt '*' file.get_managed /etc/httpd/conf.d/httpd.conf jinja salt://http/httpd.conf '{hash_type: 'md5', 'hsum': <md5sum>}' root root '755' base None None
     '''
     # If the file is a template and the contents is managed
     # then make sure to copy it down and templatize  things.
@@ -956,11 +973,11 @@ def get_managed(
         sfn = __salt__['cp.cache_file'](source, env)
         if not os.path.exists(sfn):
             return sfn, {}, 'File "{0}" could not be found'.format(sfn)
-        if template in salt.utils.templates.template_registry:
+        if template in salt.utils.templates.TEMPLATE_REGISTRY:
             context_dict = defaults if defaults else {}
             if context:
                 context_dict.update(context)
-            data = salt.utils.templates.template_registry[template](
+            data = salt.utils.templates.TEMPLATE_REGISTRY[template](
                     sfn,
                     name=name,
                     source=source,
@@ -981,8 +998,7 @@ def get_managed(
 
         if data['result']:
             sfn = data['data']
-            with salt.utils.fopen(sfn, 'r') as source:
-                hsum = hashlib.md5(source.read()).hexdigest()
+            hsum = get_hash(sfn)
             source_sum = {'hash_type': 'md5',
                           'hsum': hsum}
         else:
@@ -991,13 +1007,13 @@ def get_managed(
     else:
         # Copy the file down if there is a source
         if source:
-            if urlparse(source).scheme == 'salt':
+            if salt._compat.urlparse(source).scheme == 'salt':
                 source_sum = __salt__['cp.hash_file'](source, env)
                 if not source_sum:
                     return '', {}, 'Source file {0} not found'.format(source)
             elif source_hash:
                 protos = ['salt', 'http', 'ftp']
-                if urlparse(source_hash).scheme in protos:
+                if salt._compat.urlparse(source_hash).scheme in protos:
                     # The source_hash is a file on a server
                     hash_fn = __salt__['cp.cache_file'](source_hash)
                     if not hash_fn:
@@ -1050,6 +1066,10 @@ def check_perms(name, ret, user, group, mode):
           specify mode 0777, for example, it must be specified as the string,
           '0777' otherwise, 0777 will be parsed as an octal and you'd get 511
           instead.
+
+    CLI Example::
+
+        salt '*' file.check_perms /etc/sudoers '{}' root root 400
     '''
     if not ret:
         ret = {'name': name,
@@ -1133,6 +1153,10 @@ def check_managed(
         **kwargs):
     '''
     Check to see what changes need to be made for a file
+
+    CLI Example::
+    
+        salt '*' file.check_managed /etc/httpd/conf.d/httpd.conf salt://http/httpd.conf '{hash_type: 'md5', 'hsum': <md5sum>}' root, root, '755' jinja True None None base
     '''
     # If the source is a list then find which file exists
     source, source_hash = source_list(source, source_hash, env)
@@ -1175,7 +1199,11 @@ def check_file_meta(
         mode,
         env):
     '''
-    Check for the changes in the file metadata
+    Check for the changes in the file metadata.
+
+    CLI Example::
+
+        salt '*' file.check_file_meta /etc/httpd/conf.d/httpd.conf salt://http/httpd.conf '{hash_type: 'md5', 'hsum': <md5sum>}' root, root, '755' base
     '''
     changes = {}
     stats = __salt__['file.stats'](
@@ -1189,7 +1217,7 @@ def check_file_meta(
             if not sfn and source:
                 sfn = __salt__['cp.cache_file'](source, env)
             if sfn:
-                with nested(salt.utils.fopen(sfn, 'rb'),
+                with contextlib.nested(salt.utils.fopen(sfn, 'rb'),
                             salt.utils.fopen(name, 'rb')) as (src, name_):
                     slines = src.readlines()
                     nlines = name_.readlines()
@@ -1204,11 +1232,12 @@ def check_file_meta(
         changes['group'] = group
     # Normalize the file mode
     smode = __salt__['config.manage_mode'](stats['mode'])
-    mode  = __salt__['config.manage_mode'](mode)
+    mode = __salt__['config.manage_mode'](mode)
     if not mode is None and mode != smode:
         changes['mode'] = mode
     return changes
 
+
 def get_diff(
         minionfile,
         masterfile,
@@ -1216,7 +1245,7 @@ def get_diff(
     '''
     Return unified diff of file compared to file on master
 
-    Example:
+    CLI Example::
 
         salt \* file.get_diff /home/fred/.vimrc salt://users/fred/.vimrc
     '''
@@ -1228,7 +1257,7 @@ def get_diff(
 
     sfn = __salt__['cp.cache_file'](masterfile, env)
     if sfn:
-        with nested(salt.utils.fopen(sfn, 'r'),
+        with contextlib.nested(salt.utils.fopen(sfn, 'r'),
                     salt.utils.fopen(minionfile, 'r')) as (src, name_):
             slines = src.readlines()
             nlines = name_.readlines()
@@ -1241,6 +1270,7 @@ def get_diff(
 
     return ret
 
+
 def manage_file(name,
         sfn,
         ret,
@@ -1254,6 +1284,10 @@ def manage_file(name,
     '''
     Checks the destination against what was retrieved with get_managed and
     makes the appropriate modifications (if necessary).
+
+    CLI Example::
+
+        salt '*' file.manage_file /etc/httpd/conf.d/httpd.conf '{}' salt://http/httpd.conf '{hash_type: 'md5', 'hsum': <md5sum>}' root root '755' base ''
     '''
     if not ret:
         ret = {'name': name,
@@ -1264,10 +1298,7 @@ def manage_file(name,
     if os.path.isfile(name):
         # Only test the checksums on files with managed contents
         if source:
-            name_sum = ''
-            hash_func = getattr(hashlib, source_sum['hash_type'])
-            with salt.utils.fopen(name, 'rb') as namefile:
-                name_sum = hash_func(namefile.read()).hexdigest()
+            name_sum = get_hash(name, source_sum['hash_type'])
 
         # Check if file needs to be replaced
         if source and source_sum['hsum'] != name_sum:
@@ -1278,9 +1309,8 @@ def manage_file(name,
                     ret, 'Source file {0} not found'.format(source))
             # If the downloaded file came from a non salt server source verify
             # that it matches the intended sum value
-            if urlparse(source).scheme != 'salt':
-                with salt.utils.fopen(sfn, 'rb') as dlfile:
-                    dl_sum = hash_func(dlfile.read()).hexdigest()
+            if salt._compat.urlparse(source).scheme != 'salt':
+                dl_sum = get_hash(sfn, source_sum['hash_type'])
                 if dl_sum != source_sum['hsum']:
                     ret['comment'] = ('File sum set for file {0} of {1} does '
                                       'not match real sum of {2}'
@@ -1296,7 +1326,7 @@ def manage_file(name,
             if _is_bin(sfn) or _is_bin(name):
                 ret['changes']['diff'] = 'Replace binary file'
             else:
-                with nested(salt.utils.fopen(sfn, 'rb'),
+                with contextlib.nested(salt.utils.fopen(sfn, 'rb'),
                             salt.utils.fopen(name, 'rb')) as (src, name_):
                     slines = src.readlines()
                     nlines = name_.readlines()
@@ -1338,10 +1368,8 @@ def manage_file(name,
                     ret, 'Source file {0} not found'.format(source))
             # If the downloaded file came from a non salt server source verify
             # that it matches the intended sum value
-            if urlparse(source).scheme != 'salt':
-                hash_func = getattr(hashlib, source_sum['hash_type'])
-                with salt.utils.fopen(sfn, 'rb') as dlfile:
-                    dl_sum = hash_func(dlfile.read()).hexdigest()
+            if salt._compat.urlparse(source).scheme != 'salt':
+                dl_sum = get_hash(sfn, source_sum['hash_type'])
                 if dl_sum != source_sum['hsum']:
                     ret['comment'] = ('File sum set for file {0} of {1} does '
                                       'not match real sum of {2}'
@@ -1407,9 +1435,14 @@ def manage_file(name,
         __clean_tmp(sfn)
         return ret
 
+
 def makedirs(path, user=None, group=None, mode=None):
     '''
     Ensure that the directory containing this path is available.
+
+    CLI Example::
+
+        salt '*' file.makedirs /opt/code
     '''
     directory = os.path.dirname(path)
 
@@ -1426,10 +1459,14 @@ def makedirs(path, user=None, group=None, mode=None):
         # follow the principal of least surprise method.
 
 
-def makedirs_perms(name, user=None, group=None, mode=0755):
+def makedirs_perms(name, user=None, group=None, mode='0755'):
     '''
     Taken and modified from os.makedirs to set user, group and mode for each
     directory created.
+
+    CLI Example::
+
+        salt '*' file.makedirs_perms /opt/code
     '''
     path = os.path
     mkdir = os.mkdir
