@@ -116,7 +116,7 @@ def set_policy(table='filter', chain=None, policy=None):
     return out
 
 
-def save(filename):
+def save(filename=None):
     '''
     Save the current in-memory rules to disk
 
@@ -124,13 +124,18 @@ def save(filename):
 
         salt '*' iptables.save /etc/sysconfig/iptables
     '''
-    os.makedirs(os.path.dirname(filename))
+    if _conf() and not filename:
+        filename = _conf()
+
+    parent_dir = os.path.dirname(filename)
+    if not os.path.isdir(parent_dir):
+        os.makedirs(parent_dir)
     cmd = 'iptables-save > {0}'.format(filename)
     out = __salt__['cmd.run'](cmd)
     return out
 
 
-def append(table='filter', rule=None):
+def append(table='filter', chain=None, rule=None):
     '''
     Append a rule to the specified table/chain.
 
@@ -141,8 +146,10 @@ def append(table='filter', rule=None):
 
     CLI Example::
 
-        salt '*' iptables.append filter 'INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT'
+        salt '*' iptables.append filter INPUT '-m state --state RELATED,ESTABLISHED -j ACCEPT'
     '''
+    if not chain:
+        return 'Error: Chain needs to be specified'
     if not rule:
         return 'Error: Rule needs to be specified'
 
@@ -151,7 +158,7 @@ def append(table='filter', rule=None):
     return out
 
 
-def insert(table='filter', rule=None):
+def insert(table='filter', chain=None, position=None, rule=None):
     '''
     Insert a rule into the specified table/chain, at the specified position.
 
@@ -162,9 +169,11 @@ def insert(table='filter', rule=None):
 
     CLI Examples::
 
-        salt '*' iptables.insert filter 'INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT'
-        salt '*' iptables.insert filter 'INPUT 3 -m state --state RELATED,ESTABLISHED -j ACCEPT'
+        salt '*' iptables.insert filter INPUT rule='-m state --state RELATED,ESTABLISHED -j ACCEPT'
+        salt '*' iptables.insert filter INPUT position=3 rule='-m state --state RELATED,ESTABLISHED -j ACCEPT'
     '''
+    if not chain:
+        return 'Error: Chain needs to be specified'
     if not rule:
         return 'Error: Rule needs to be specified'
 
@@ -173,7 +182,7 @@ def insert(table='filter', rule=None):
     return out
 
 
-def delete(table, position=None, rule=None):
+def delete(table, chain, position=None, rule=None):
     '''
     Delete a rule from the specified table/chain, specifying either the rule
         in its entirety, or the rule's position in the chain.
@@ -185,8 +194,8 @@ def delete(table, position=None, rule=None):
 
     CLI Examples::
 
-        salt '*' iptables.delete filter 3
-        salt '*' iptables.delete filter rule='INPUT 3 -m state --state RELATED,ESTABLISHED -j ACCEPT'
+        salt '*' iptables.delete filter INPUT position=3
+        salt '*' iptables.delete filter INPUT rule='-m state --state RELATED,ESTABLISHED -j ACCEPT'
     '''
 
     cmd = ''
@@ -248,6 +257,7 @@ def _parse_conf(conf_file=None, in_mem=False):
             ret[table][chain]['packet count'] = pcount
             ret[table][chain]['byte count'] = bcount
             ret[table][chain]['rules'] = []
+            ret[table][chain]['rules_comment'] = {}
         elif line.startswith('-A'):
             parser = _parser()
             parsed_args = []
@@ -261,6 +271,9 @@ def _parse_conf(conf_file=None, in_mem=False):
             for arg in parsed_args:
                 if parsed_args[arg] and arg is not 'append':
                     ret_args[arg] = parsed_args[arg]
+            if parsed_args['comment'] is not None:
+                comment = parsed_args['comment'][0].strip('"')
+                ret[table][chain[0]]['rules_comment'][comment]=ret_args
             ret[table][chain[0]]['rules'].append(ret_args)
     return ret
 
