@@ -3,10 +3,11 @@ Module for gathering and managing network information
 '''
 
 # Import python libs
+import re
 import logging
 
 # Import salt libs
-from salt.utils.socket_util import sanitize_host
+import salt.utils.socket_util
 
 
 log = logging.getLogger(__name__)
@@ -70,7 +71,6 @@ def _interfaces_ip(out):
     Uses ip to return a dictionary of interfaces with various information about
     each (up/down state, ip address, netmask, and hwaddr)
     '''
-    import re
     ret = dict()
 
     def parse_network(value, cols):
@@ -160,7 +160,6 @@ def _interfaces_ifconfig(out):
     Uses ifconfig to return a dictionary of interfaces with various information
     about each (up/down state, ip address, netmask, and hwaddr)
     '''
-    import re
     ret = dict()
 
     piface = re.compile('^(\S+):?')
@@ -240,6 +239,17 @@ def interfaces():
     return ifaces
 
 
+def hwaddr(iface):
+    '''
+    Return the hardware address (a.k.a. MAC address) for a given interface
+
+    CLI Example::
+
+        salt '*' network.hwaddr eth0
+    '''
+    return interfaces().get(iface, {}).get('hwaddr', '')
+
+
 def _get_net_start(ipaddr, netmask):
     ipaddr_octets = ipaddr.split('.')
     netmask_octets = netmask.split('.')
@@ -271,6 +281,10 @@ def _ipv4_to_bits(ipaddr):
 def subnets():
     '''
     Returns a list of subnets to which the host belongs
+
+    CLI Example::
+
+        salt '*' network.subnets
     '''
     ifaces = interfaces()
     subnets = []
@@ -287,6 +301,10 @@ def subnets():
 def in_subnet(cidr):
     '''
     Returns True if host is within specified subnet, otherwise False
+
+    CLI Example::
+
+        salt '*' network.in_subnet 10.0.0.0/16
     '''
     try:
         netstart, netsize = cidr.split('/')
@@ -319,6 +337,10 @@ def ip_addrs(interface=None, include_loopback=False):
     Returns a list of IPv4 addresses assigned to the host. 127.0.0.1 is
     ignored, unless 'include_loopback=True' is indicated. If 'interface' is
     provided, then only IP addresses from that interface will be returned.
+
+    CLI Example::
+
+        salt '*' network.ip_addrs
     '''
     ret = []
     ifaces = interfaces()
@@ -342,6 +364,10 @@ def ip_addrs6(interface=None, include_loopback=False):
     Returns a list of IPv6 addresses assigned to the host. ::1 is ignored,
     unless 'include_loopback=True' is indicated. If 'interface' is provided,
     then only IP addresses from that interface will be returned.
+
+    CLI Example::
+
+        salt '*' network.ip_addrs6
     '''
     ret = []
     ifaces = interfaces()
@@ -368,7 +394,7 @@ def ping(host):
 
         salt '*' network.ping archlinux.org
     '''
-    cmd = 'ping -c 4 {0}'.format(sanitize_host(host))
+    cmd = 'ping -c 4 {0}'.format(salt.utils.socket_util.sanitize_host(host))
     return __salt__['cmd.run'](cmd)
 
 
@@ -421,7 +447,7 @@ def traceroute(host):
         salt '*' network.traceroute archlinux.org
     '''
     ret = []
-    cmd = 'traceroute {0}'.format(sanitize_host(host))
+    cmd = 'traceroute {0}'.format(salt.utils.socket_util.sanitize_host(host))
     out = __salt__['cmd.run'](cmd)
 
     for line in out:
@@ -452,5 +478,5 @@ def dig(host):
 
         salt '*' network.dig archlinux.org
     '''
-    cmd = 'dig {0}'.format(sanitize_host(host))
+    cmd = 'dig {0}'.format(salt.utils.socket_util.sanitize_host(host))
     return __salt__['cmd.run'](cmd)
