@@ -179,6 +179,10 @@ def create_ca(
     the resulting CA would be written in the following location::
 
     /etc/pki/koji/koji_ca_cert.crt
+
+    CLI Example::
+
+        salt '*' tls.create_ca test_ca
     '''
     if _ca_exists(ca_name):
         return 'Certificate for CA named "{0}" already exists'.format(ca_name)
@@ -310,6 +314,10 @@ def create_csr(
 
     /etc/pki/koji/certs/test.egavas.org.csr
     /etc/pki/koji/certs/test.egavas.org.key
+
+    CLI Example::
+
+        salt '*' tls.create_csr test
     '''
 
     if not _ca_exists(ca_name):
@@ -422,6 +430,10 @@ def create_self_signed_cert(
 
     /etc/pki/tls/certs/test.egavas.org.crt
     /etc/pki/tls/certs/test.egavas.org.key
+
+    CLI Example::
+
+        salt '*' tls.create_self_signed_cert
     '''
 
     if not os.path.exists('{0}/{1}/certs/'.format(_cert_base_path(), tls_dir)):
@@ -501,7 +513,7 @@ def create_ca_signed_cert(ca_name, CN, days=365):
     ca_name
         name of the CA
     CN
-        common name matching the the certificate signing request
+        common name matching the certificate signing request
     days
         number of days certficate is valid, default is 365 (1 year)
 
@@ -510,6 +522,10 @@ def create_ca_signed_cert(ca_name, CN, days=365):
 
     The CN *must* match an existing CSR generated by create_csr. If it
     does not, this method does nothing.
+
+    CLI Example::
+
+        salt '*' tls.create_ca_signed_cert test localhost
     '''
     if os.path.exists(
             '{0}/{1}/{2}.crt'.format(_cert_base_path(), ca_name, CN)
@@ -590,6 +606,10 @@ def create_pkcs12(ca_name, CN, passphrase=''):
         common name matching the the certificate signing request
     passphrase
         used to unlock the PKCS#12 certificate when loaded into the browser
+
+    CLI Example::
+
+        salt '*' tls.create_pkcs12 test localhost
     '''
     if os.path.exists(
             '{0}/{1}/certs/{2}.p12'.format(
