@@ -19,13 +19,16 @@ import logging
 try:
     import win32security
     import ntsecuritycon as con
-    has_windows_modules = True
+    HAS_WINDOWS_MODULES = True
 except ImportError:
-    has_windows_modules = False
+    HAS_WINDOWS_MODULES = False
 
 # Import salt libs
 import salt.utils
 from salt.exceptions import SaltInvocationError
+from file import check_hash, check_managed, check_perms, contains_regex,\
+        directory_exists, get_managed, makedirs, makedirs_perms, manage_file,\
+        patch, remove, source_list, stats
 
 
 log = logging.getLogger(__name__)
@@ -35,8 +38,8 @@ def __virtual__():
     '''
     Only works on Windows systems
     '''
-    if __grains__['os'] == 'Windows':
-        if has_windows_modules:
+    if salt.utils.is_windows():
+        if HAS_WINDOWS_MODULES:
             return 'file'
         log.warn(salt.utils.required_modules_error(__file__, __doc__))
     return False
@@ -193,7 +196,7 @@ def chown(path, user, group):
         salt '*' file.chown c:\\temp\\test.txt myusername administrators
     '''
     # I think this function isn't working correctly yet
-    sd = win32security.GetFileSecurity(
+    gsd = win32security.GetFileSecurity(
         path, win32security.DACL_SECURITY_INFORMATION
     )
     uid = user_to_uid(user)
@@ -217,9 +220,9 @@ def chown(path, user, group):
         win32security.ACL_REVISION, con.FILE_ALL_ACCESS,
         win32security.GetBinarySid(gid)
     )
-    sd.SetSecurityDescriptorDacl(1, dacl, 0)
+    gsd.SetSecurityDescriptorDacl(1, dacl, 0)
     return win32security.SetFileSecurity(
-        path, win32security.DACL_SECURITY_INFORMATION, sd
+        path, win32security.DACL_SECURITY_INFORMATION, gsd
     )
 
 
@@ -259,14 +262,14 @@ def get_sum(path, form='md5'):
         return getattr(hashlib, form)(
             salt.utils.fopen(path, 'rb').read()
         ).hexdigest()
-    except (IOError, OSError) as e:
-        return 'File Error: {0}'.format(e)
+    except (IOError, OSError) as err:
+        return 'File Error: {0}'.format(err)
     except AttributeError:
         return 'Hash {0} not supported'.format(form)
     except NameError:
         return 'Hashlib unavailable - please fix your python install'
-    except Exception as e:
-        return str(e)
+    except Exception as err:
+        return str(err)
 
 
 def find(path, **kwargs):
@@ -361,20 +364,20 @@ def find(path, **kwargs):
         salt '*' file.find /var/log name=\*.[0-9] mtime=+30d size=+10m delete
     '''
     try:
-        f = salt.utils.find.Finder(kwargs)
+        finder = salt.utils.find.Finder(kwargs)
     except ValueError as ex:
         return 'error: {0}'.format(ex)
 
-    ret = [p for p in f.find(path)]
+    ret = [p for p in finder.find(path)]
     ret.sort()
     return ret
 
 
-def _sed_esc(s):
+def _sed_esc(string):
     '''
     Escape single quotes and forward slashes
     '''
-    return '{0}'.format(s).replace("'", "'\"'\"'").replace("/", "\/")
+    return '{0}'.format(string).replace("'", "'\"'\"'").replace("/", "\/")
 
 
 def sed(path, before, after, limit='', backup='.bak', options='-r -e',
@@ -542,9 +545,9 @@ def append(path, *args):
     '''
     # Largely inspired by Fabric's contrib.files.append()
 
-    with salt.utils.fopen(path, "a") as f:
+    with salt.utils.fopen(path, "a") as ofile:
         for line in args:
-            f.write('{0}\n'.format(line))
+            ofile.write('{0}\n'.format(line))
 
     return "Wrote {0} lines to '{1}'".format(len(args), path)
 
