@@ -129,6 +129,10 @@ class SREQ(object):
             self.socket.setsockopt(
                 zmq.RECONNECT_IVL_MAX, 5000
             )
+        if hasattr(zmq, 'IPV4ONLY'):
+            self.socket.setsockopt(
+                zmq.IPV4ONLY, 0
+            )
         self.socket.linger = linger
         if id_:
             self.socket.setsockopt(zmq.IDENTITY, id_)
@@ -156,10 +160,7 @@ class SREQ(object):
                         timeout * tried
                     )
                 )
-        try:
-            return self.serial.loads(self.socket.recv())
-        finally:
-            self.poller.unregister(self.socket)
+        return self.serial.loads(self.socket.recv())
 
     def send_auto(self, payload):
         '''
