@@ -27,6 +27,34 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         self.assertIn('minion', data)
         self.assertIn('sub_minion', data)
 
+    def test_compound(self):
+        '''
+        test salt compound matcher
+        '''
+        data = self.run_salt('-C "min* and G@test_grain:cheese" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertNotIn('sub_minion', data)
+        data = self.run_salt('-C "min* and not G@test_grain:foo" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertNotIn('sub_minion', data)
+        data = self.run_salt('-C "min* not G@test_grain:foo" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertNotIn('sub_minion', data)
+        match = 'P@test_grain:^cheese$ and * and G@test_grain:cheese'
+        data = self.run_salt('-t 1 -C \'{0}\' test.ping'.format(match))
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertNotIn('sub_minion', data)
+        match = 'L@sub_minion and E@.*'
+        data = self.run_salt('-t 1 -C "{0}" test.ping'.format(match))
+        data = '\n'.join(data)
+        self.assertIn('sub_minion', data)
+        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
+
+
     def test_glob(self):
         '''
         test salt glob matcher
@@ -57,6 +85,7 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         '''
         test salt grain matcher
         '''
+        # First-level grain (string value)
         data = self.run_salt('-t 1 -G "test_grain:cheese" test.ping')
         data = '\n'.join(data)
         self.assertIn('minion', data)
@@ -65,6 +94,33 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         data = '\n'.join(data)
         self.assertIn('sub_minion', data)
         self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
+        # First-level grain (list member)
+        data = self.run_salt('-t 1 -G "planets:earth" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertNotIn('sub_minion', data)
+        data = self.run_salt('-G "planets:saturn" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('sub_minion', data)
+        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
+        # Nested grain (string value)
+        data = self.run_salt('-t 1 -G "level1:level2:foo" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertNotIn('sub_minion', data)
+        data = self.run_salt('-G "level1:level2:bar" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('sub_minion', data)
+        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
+        # Nested grain (list member)
+        data = self.run_salt('-t 1 -G "companions:one:ian" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertNotIn('sub_minion', data)
+        data = self.run_salt('-G "companions:two:jamie" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('sub_minion', data)
+        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
 
     def test_regrain(self):
         '''
@@ -85,33 +141,31 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         '''
         test pillar matcher
         '''
+        # First-level pillar (string value)
         data = self.run_salt('-I "monty:python" test.ping')
         data = '\n'.join(data)
         self.assertIn('minion', data)
         self.assertIn('sub_minion', data)
+        # First-level pillar (string value, only in sub_minion)
         data = self.run_salt('-I "sub:sub_minion" test.ping')
         data = '\n'.join(data)
         self.assertIn('sub_minion', data)
         self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
+        # First-level pillar (list member)
         data = self.run_salt('-I "knights:Bedevere" test.ping')
         data = '\n'.join(data)
         self.assertIn('minion', data)
         self.assertIn('sub_minion', data)
-
-    def test_compound(self):
-        '''
-        test compound matcher
-        '''
-        match = 'P@test_grain:^cheese$ and * and G@test_grain:cheese'
-        data = self.run_salt('-t 1 -C \'{0}\' test.ping'.format(match))
+        # Nested pillar (string value)
+        data = self.run_salt('-I "level1:level2:foo" test.ping')
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertIn('sub_minion', data)
+        # Nested pillar (list member)
+        data = self.run_salt('-I "companions:three:sarah jane" test.ping')
         data = '\n'.join(data)
         self.assertIn('minion', data)
-        self.assertNotIn('sub_minion', data)
-        match = 'L@sub_minion and E@.*'
-        data = self.run_salt('-t 1 -C "{0}" test.ping'.format(match))
-        data = '\n'.join(data)
         self.assertIn('sub_minion', data)
-        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
 
     def test_exsel(self):
         data = self.run_salt('-X test.ping test.ping')
@@ -119,7 +173,7 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         self.assertIn('minion', data)
         self.assertIn('sub_minion', data)
 
-    def test_ipcadr(self):
+    def test_ipcidr(self):
         subnets_data = self.run_salt('--out yaml \'*\' network.subnets')
         yaml_data = yaml.load('\n'.join(subnets_data))
 
@@ -143,7 +197,7 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         '''
         Test to see if we're supporting --doc
         '''
-        data = self.run_salt('-d user.add')
+        data = self.run_salt('-d user')
         self.assertIn('user.add:', data)
 
     def test_salt_documentation_arguments_not_assumed(self):
@@ -154,15 +208,14 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         self.assertIn('user.add:', data)
         data = self.run_salt('\'*\' -d')
         self.assertIn('user.add:', data)
-        data = self.run_salt('\'*\' -d user.add')
+        data = self.run_salt('\'*\' -d user')
         self.assertIn('user.add:', data)
-        data = self.run_salt('\'*\' sys.doc -d user.add')
+        data = self.run_salt('\'*\' sys.doc -d user')
         self.assertIn('user.add:', data)
-        data = self.run_salt('\'*\' sys.doc user.add')
+        data = self.run_salt('\'*\' sys.doc user')
         self.assertIn('user.add:', data)
 
 
-
 if __name__ == "__main__":
     loader = TestLoader()
     tests = loader.loadTestsFromTestCase(MatchTest)
